
    // LocationFile.hpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#ifndef __LOCATIONFILE_HPP__
#define __LOCATIONFILE_HPP__

#include <iostream>
#include <fstream>

using namespace std;

#include "Terminal.hpp"
#include "LocationBuilder.hpp"

using namespace Terminal;

#define LF_ENDFILE "End"
#define LF_LOCATION "location"
#define LF_START "start"
#define LF_LIGHT "light"

#define LF_PATH "../worlds/"

namespace Loaders{

  /**
     I help in getting data from a text file in a specified format(A Location File!).

     @section Usage

     You have to use my interface as in this example:

     @code

     ...
     LocationFile f("/any/path/you/like/location001");

     std::string type;
     stringc name;
     vector3df start;

     f.open_read();
   
     type = f.next()
     if (type == LF_LOCATION){
     name = f.get_name();
     start = f.get_start();
     }

     f.close();

     ...
     @endcode

     @section File Format
     The file must be a text file like this:

     @verbatim
     Begin location
     name of this location
     model_name
     texture_name
     Begin start
     0.0 1.0 0.0
     Begin light
     0.0 -100.0 0.0
     1.0 1.0 1.0 1.0
     360.0
     @endverbatim
   
     @todo Removing the constructor Location(std::string filename) will lead on removing lot of instance variables and hence will save a lot of memory and simplify class implementation.

  */
  class LocationFile {
  
  protected:
    /**
       The path to the file for writing or reading.
    */
    string filename;

    ifstream file;

    string next_type;

    /**
       File has been opened?     
    */
    bool opened;

    
    /**
       @name Light cache data.
     */
    //@{
    /**
       The light name cache.
     */
    string lname;
    /**
       The light direction cache.
     */
    float ldir[3];
    /**
       The light color cache.
     */
    float lcolor[4];
    // float lradius;
    //@}

    // vector3df startpos;
    string name;

    string model;
    // string texture;

    LocationBuilder *builder;
  
  public:

    /**
       Create an instance of LocationFile using the file path from filename.

       @deprecated This constructor is deprecated.

       @param filename An instance of a standard string that has the path of the file to open or write.
    */
    LocationFile(std::string filename);

    /**
       Create an instance of LocationFile using a builder. This will be usefull for loading wile creating the Location.

       I create the filename according to the worldname and location name.

       @param builder a LocationBuilder instance that will construct the Location.
       @param worldname An instance of std::string for the worldname.
       @param loc_name An instance of std::string that gives the location name.
    */
    LocationFile(LocationBuilder *builder, std::string worldname, std::string loc_name);
    ~LocationFile();
  
    /**
       Retrieve next element type. Get ready all the data of that type.
     
       Use LF_* constants for comparing the types.

       @return An instance of string that has the format of the next element according to the loaded file.
       @retval An instance of string with an empty string if something goes wrong.
       @retval An instance of string with LF_ENDFILE string if file was not opened.
    */
    string next();

    /**
       You'll get the light direction loaded from me.
     
       @return A float array with three elements.
    */
    float* get_light_dir();
    /**
       I give the light color loaded from me.

       @return An instance of SColorf.
    */
    float* get_light_color();
    /**
       I give the light name.

       @return An instance of string.
    */
    string get_light_name();
    
  
    /**
       I give you the starting position.

       @return an instance of vector3df.
    */ 
    // vector3df get_starting_pos();
  
    /**
       I give the name of the location that is stored inside the file.
       @return an instance of string.
    */
    string get_name();


    /**
       I give the model name I have loaded from the file.
       @return an instance of string with the model name.
    */
    string get_model_name();

    /**
       I give the texture name I have loaded from the file.
       @return an instance of string.
    */
    // string get_texture_name();

    /**
       Tell me that the file is for reading only.
    */
    void open_read();
    /**
       You don't need the file any more? I'll close it! :)
    */
    void close();

    /**
       Load all the file using the builder.

       A LocationBuilder must be setted to use this functionality.    
    */
    void load_file();

  protected:
    bool load_light_dir();
    bool load_light_color();
    bool load_light_name();

    // void load_starting_pos();
  
    void load_name();
    void load_model_name();
    // void load_texture_name();

    /**
       This method is used inside the next() method.      

       Load the next type element according to next_type variable retrieving information from the file.

       Send the builder(if there is one setted) the build_* message depending on what it has been loaded.
    */
    void load_element();
  };
}

#endif
